#####################################################################
# Defines all Blender user interface panel and RNA variables
#####################################################################

from bpy import *
from .core import *
from . import store
from . import helpers

# some tmp variables the UI script uses
importPathOld = ""
viewFilterOld = "4"
mappingOld = 0
activeobjOld = ""
filePathOld = ""
particleDensityOld = 0
importReady = False

# Make aliases for a few commonly used bpy types
StringOBJ = props.StringProperty
String = props.StringProperty
Float = props.FloatProperty
Int = props.IntProperty
Bool = props.BoolProperty
Enum = props.EnumProperty

# Create RNA variables for the UI Panel to use
types.Object.BBInfo = StringOBJ(attr="BBInfo", description="Information on each atom", default="")

types.Scene.BBImportPath = String(attr="BBImportPath", description="PDB import file path, or a 4-character PDB.org code", default="", subtype="FILE_PATH")
types.Scene.BBModelRemark = String(attr="BBModelRemark",  description="Model name tag for multiple imports", default="protein0")
types.Scene.BBImportFeedback = String(attr="BBImportFeedback", description="", default="")
types.Scene.BBImportChain = String(attr="BBImportChain", description="", default="")
types.Scene.BBImportOrder = String(attr="BBImportOrder", description="List of models to be imported", default="")
types.Scene.BBDeltaFrame = Int (attr="BBDeltaFrame", name="Keyframe Interval", description="The number of in-between frames between each model for animation", default=100, min=1, max=200, soft_min=5, soft_max=50)
types.Scene.BBVerbose = Bool(attr="BBVerbose", name="Verbose", description="Be verbose for debugging", default=False)
types.Scene.BBSpaceFill = Bool(attr="BBSpaceFill", name="SpaceFill", description="Enable to draw Van der Waals radii, disable to draw covalent radii", default=False)
types.Scene.BBSideChain = Bool(attr="BBSideChain", name="SideChain", description="Import sidechain (slower)", default=True)
types.Scene.BBImportHydrogen = Bool(attr="BBImportHydrogen", name="Hydrogen", description="Import hydrogen atoms (Slower)", default=False)
types.Scene.BBMakeBonds = Bool(attr="BBMakeBonds", name="Make Bonds", description="Generate physic onstrains using atomic bonds (Slower)", default=False)
types.Scene.BBHighQuality = Bool(attr="BBHighQuality", name="High Quality", description="Generate high-Poly mesh (Slower)", default=False)
types.Scene.BBSingleUser = Bool(attr="BBSingleUser", name="Single User", description="Force single user mesh (Slower)", default=False)
types.Scene.BBReportError = Bool(attr="BBReportError", name="Upload Errors", description="Upload errors to developer, this helps in making the program better", default=False)

types.Scene.BBDeltaPhysicRadius = Float(attr="BBDeltaPhysicRadius", name="Radius", description="Value of radius for collisions in physics engine", default=0.7, min=0.1, max=2, soft_min=.1, soft_max=2)

types.Scene.BBSolveMotion = Int(attr="BBSolveMotion", name="Collision", description="Use the physics engine to calculate protein motion (0 is off, 1 is on, 2 is recording)", default=0, min=0, max=2, soft_min=0, soft_max=2)
types.Scene.BBMLPSolventRadius = Float(attr="BBMLPSolventRadius", name="Solvent Radius", description="Solvent Radius used for Surface Generation", default=1.4, min=0.2, max=5, soft_min=0.4, soft_max=4)

types.Scene.BBMLPGridSpacing = Float(attr="BBMLPGridSpacing", name="Grid Spacing", description="MLP Calculation step size (Smaller is better, but slower)", default=1, min=1, max=20, soft_min=1.4, soft_max=10)
types.Scene.BBMLPContrast = Float(attr="BBMLPContrast", name="Contrast", description="Contrast of the mapped MLP value", default=1.0, min=0.0, max=5, soft_min=0.3, soft_max=3)
types.Scene.BBMLPBrightness = Float(attr="BBMLPBrightness", name="Brightness", description="Brightness offset of the mapped MLP value", default=0.0, min=-10, max=10, soft_min=-3, soft_max=3)

types.Scene.BBEPIonConc = Float(attr="BBEPIonConc", name="Ion concentration", description="Ion concentration of the solvent", default=0.15, min=0.01, max=1, soft_min=0.01, soft_max=1)
types.Scene.BBEPGridStep = Float(attr="BBEPGridStep", name="Grid Spacing", description="EP Calculation step size (Smaller is better, but slower)", default=1, min=0.3, max=10, soft_min=0.5, soft_max=5)
types.Scene.BBEPMinPot = Float(attr="BBEPMinPot", name="Minimum Potential", description="Minimum Potential on the surface from which start the calculation of the field lines", default=0.0, min=0.0, max=10000, soft_min=0, soft_max=1000)
types.Scene.BBEPNumOfLine = Float(attr="BBEPNumOfLine", name="n EP Lines*eV/Å² ", description="Concentration of lines", default=0.05, min=0.01, max=0.5, soft_min=0.01, soft_max=0.1, precision=3, step=0.01)
types.Scene.BBEPParticleDensity = Float(attr="BBEPParticleDensity", name="Particle Density", description="Particle Density", default=1, min=0.1, max=10.0, soft_min=0.1, soft_max=5.0)
types.Scene.BBExportStep = Int (attr="BBExportStep", name="Export Step", description="Export step", default=10, min=1, max=100, soft_min=1, soft_max=50)

types.Scene.BBNMANbModel = Int (attr="BBNMANbModel", name="NMA steps", description="Number of conformations to be calculated in each direction", default=6, min=1, max=100, soft_min=1, soft_max=50)
types.Scene.BBNMARMSD = Float(attr="BBNMARMSD", name="RMSD sampling", description="RMSD between the given and the farthest conformation", default=0.8, min=0.1, max=5.0, soft_min=0.1, soft_max=5.0, precision=2, step=1.0)
types.Scene.BBNMACutoff = Float (attr="BBNMACutoff", name="NMA cutoff", description="NMA cutoff distance (Å) for pairwise interactions", default=15.0, min=0.0, max=25.0, soft_min=1.0, soft_max=25.0)
types.Scene.BBNMAGamma = Float(attr="BBNMAGamma", name="NMA Gamma", description="NMA spring constant", default=1.0, min=0.0, max=10.0, soft_min=0.1, soft_max=5.0, precision=2, step=1.0)

types.Scene.BBViewFilter = Enum(attr="BBViewFilter", name="View Filter", description="Select a view mode",
	items=(("1", "Main Chain", ""),
		("2", "+ Side Chain", ""),
		("3", "+ Hydrogen", ""),
		("4", "Surface", "")),
	default="3")

types.Scene.BBMLPFormula = Enum(attr="BBMLPFormula", name="Formula", description="Select a formula for MLP calculation",
	items=(("0", "Dubost", ""),
		("1", "Testa", ""),
		("2", "Fauchere", ""),
		("3", "Brasseur", ""),
		("4", "Buckingham", "")),
	default="1")

types.Scene.BBForceField = Enum(attr="BBForceField", name="ForceField", description="Select a forcefield type for EP calculation",
items=(("0", "amber", ""),
	("1", "charmm", ""),
	("2", "parse", ""),
	("3", "tyl06", ""),
	("4", "peoepb", ""),
	("5", "swanson", "")),
default="0")

types.Scene.BBNormalModeAnalysis = Enum(attr="BBNormalModeAnalysis", name="Mode", description="Select a normal mode analysis to show",
items=(("0", "1", ""),
	("1", "2", ""),
	("2", "3", ""),
	("3", "4", ""),
	("4", "5", ""),
	("5", "6", ""),
	("6", "7", ""),
	("7", "8", ""),
	("8", "9", ""),
	("9", "10", ""),
	("10", "11", ""),
	("11", "12", ""),
	("12", "13", ""),
	("13", "14", ""),
	("14", "15", ""),	
	("15", "16", ""),	
	("16", "17", ""),	
	("17", "18", ""),	
	("18", "19", ""),	
	("19", "20", "")),
default="0")


types.Scene.BBRender = Enum(attr="BBRender", name="Visualize", description="Select a visualization to render",
	items=(("0", "Atom", ""),
		("1", "Plain Surface", ""),
		("2", "MLP", ""),
		("3", "EP + Plain Surface", ""),
		("4", "EP + MLP", "")),
	default="4")



class BBPanel():
	bl_space_type = 'PROPERTIES'
	bl_region_type = 'WINDOW'
	bl_context = "scene"
	@classmethod
	def poll(cls, context):
		try:
			# prevent the user from doing anything silly with the template file
			# by immediately save the file as a different file.
			# this means the only way to edit template.blend is by commenting out the following codes
			if os.path.basename(context.blend_data.filepath) == "template.blend":
				dir = os.path.dirname(context.blend_data.filepath)
				name = "untitled.blend"
				ops.wm.save_as_mainfile(filepath=os.path.join(dir, name))
			
			# Monitors certain properties for change,
			# and invoke certain functions, since Blender API has no callback functions

			#Run file preview everytime BBImportPath is changed
			global importPathOld
			global importReady
			if context.scene.BBImportPath != importPathOld:
				importPathOld = context.scene.BBImportPath
				importReady = importPreview()

			# re-save the cache if the file is saved-as under a different name
			global filePathOld
			if context.blend_data.filepath != filePathOld:
				filePathOld = context.blend_data.filepath
				if store.modelContainer:
					sessionSave()

			# do a view update if the viewFilter enumuerator is changed
			global viewFilterOld
			if context.scene.BBViewFilter != viewFilterOld:
				viewFilterOld = context.scene.BBViewFilter
				updateView()

			# do a MLP update if the either brightness or contrast is changed
			global mappingOld
			combined = context.scene.BBMLPContrast+context.scene.BBMLPBrightness
			if combined != mappingOld:
				mappingOld = combined
				mlp(force = False)

			# do a view update when the selected/active obj changes
			global activeobjOld
			if context.active_object != activeobjOld:
				# get the tag of the active model
				if context.active_object:
					tag = context.active_object.name.split("#")[0]
					# load previous sessions from cache
					if not store.modelContainer:
						sessionLoad()
						print("Sessionload")
					# set active tag
					if tag in store.modelContainer:
						store.activeTag = tag
				activeobjOld = context.active_object
				updateView(residue = context.active_object)

			# set particle density for EP visualization
			global particleDensityOld
			if context.scene.BBEPParticleDensity != particleDensityOld:
				particleDensityOld = context.scene.BBEPParticleDensity
				ob = select("Emitter")
				if ob:
					density = int(context.scene.BBEPNumOfLine * 50000 * context.scene.BBEPParticleDensity)
					ob.particle_systems[0].settings.count = density
					if not context.screen.is_animation_playing:
						ops.screen.animation_play()
		except Exception as E:
			s = "Context Poll Failed: " + str(E)
			if s.find("BBView")==-1:
				print (s)
				errorNotifier(text=s)
		# check for the correct context
		return (context.scene)

class BBPanelFile(BBPanel, types.Panel):
	bl_label = "BioBlender Select PDB File"
	def draw(self, context):
		scene = context.scene
		layout = self.layout
		split = layout.split(percentage=0.7)
		split.prop(scene, "BBImportPath", text="")
		split.prop(scene, "BBModelRemark", text="")


class BBPanelImport(BBPanel, types.Panel):
	bl_label = "BioBlender Import"
	def draw(self, context):
		scene = context.scene
		layout = self.layout
		split = layout.split(percentage=0.7)
		c = split.column()
		if getNumModel() != 1:
			c.prop(scene, "BBImportOrder", text="")
		c.prop(scene, "BBImportFeedback", text="", emboss=False)
		c.prop(scene, "BBImportChain", text="", emboss=False)
		r = c.row()
		r.scale_y = 2
		if importReady:
			r.operator("BBOperatorImport")
		else:
			r.active = False
			r.operator("BBOperatorImport", text="Error: Not Ready to Import", icon="X")

		# right column
		c = split.column()
		if getNumModel() != 1:
			c.prop(scene, "BBDeltaFrame")
		c.prop(scene, "BBVerbose")
		c.prop(scene, "BBSpaceFill")
		c.prop(scene, "BBImportHydrogen")
		c.prop(scene, "BBMakeBonds")
		c.prop(scene, "BBHighQuality")
		c.prop(scene, "BBSingleUser")
		c.prop(scene, "BBReportError")




class BBPanelView(BBPanel, types.Panel):
	bl_label = "BioBlender View"
	def draw(self, context):
		scene = context.scene
		layout = self.layout

		r = layout.column()
		r.label("Currently Selected Model: %s" % store.activeTag)
		if context.active_object:
			r.prop(context.active_object, "BBInfo", icon="MATERIAL", emboss=False)

		split = layout.split(percentage=0.75)
		r = split.row()
		r.prop(scene, "BBViewFilter", expand=True)

		split = split.row(align=True)
		split.prop(scene, "BBMLPSolventRadius")
		split.operator("BBOperatorSurface", text="", icon="FILE_REFRESH")

		
		r = layout.row(align = True)
		#split=layout.split(percentage=0.50)
		split = r.split(percentage=0.40)
		split.operator("BBOperatorInteractive")
		split.prop(scene, "BBSolveMotion")
		split.prop(scene, "BBDeltaPhysicRadius")
		
		split = split.row(align=True)
		#split.prop(scene, "BBAtomicMLP")
		split.operator("BBOperatorAtomicMLP")




class BBPanelMLP(BBPanel, types.Panel):
	bl_label = "BioBlender MLP Visualization"
	def draw(self, context):
		scene = context.scene
		layout = self.layout
		split = layout.split()

		c = split.column()
		c.prop(scene, "BBMLPFormula")
		c.prop(scene, "BBMLPGridSpacing")
		r = split.row()
		r.scale_y = 2
		r.operator("BBOperatorMLP")
		split = layout.split()
		r = split.column(align=True)
		r.prop(scene, "BBMLPContrast")
		r.prop(scene, "BBMLPBrightness")

		r = split.column()
		r.scale_y = 2
		r.operator("BBOperatorMLPRender")


class BBPanelEP(BBPanel, types.Panel):
	bl_label = "BioBlender EP Visualization"
	def draw(self, context):
		scene = context.scene
		layout = self.layout
		split = layout.split()

		c = split.column()
		c.prop(scene, "BBForceField")
		c = c.column(align=True)
		c.label("Options:")
		c.prop(scene, "BBEPIonConc")
		c.prop(scene, "BBEPGridStep")
		c.prop(scene, "BBEPMinPot")
		c.prop(scene, "BBEPNumOfLine")
		c.prop(scene, "BBEPParticleDensity")

		c = split.column()
		c.scale_y = 2
		c.operator("BBOperatorEP")
		c.operator("BBOperatorEPClear")
		
class BBPanelNMA(BBPanel, types.Panel):
	bl_label = "BioBlender NMA Visualization"
	def draw(self, context):
		scene = context.scene
		layout = self.layout
		split = layout.split()

		c = split.column()
		c.prop(scene, "BBNormalModeAnalysis")
		c.label("Options:")
		c.prop(scene, "BBNMANbModel")
		c.prop(scene, "BBNMARMSD")
		c.prop(scene, "BBNMACutoff")
		c.prop(scene, "BBNMAGamma")
		
		c = c.column(align=True)
		
		c = split.column()
		c.scale_y = 2
		c.operator("BBOperatorNMATrajectories")
		#c.operator("BBOperatorNMAVectors")	
		
class BBPanelAnimation(BBPanel, types.Panel):
	bl_label = "BioBlender Output"

	def draw(self, context):
		scene = context.scene
		layout = self.layout

		r = layout.row()
		r.prop(scene.render,"filepath", text="")

		split = layout.split()

		left = split.column()
		left.prop(scene, "BBRender")
		left.prop(scene.render,"stamp_note_text", text="Notes")
		left.prop(scene.render,"use_stamp", text ="Information Overlay")
		left.prop(scene.world.light_settings, "use_environment_light", text="Ambient Light")

		right = split.column()
		c = right.column(align=True)
		c.prop(scene, "frame_start")
		c.prop(scene, "frame_end")
		c.prop(scene, "BBExportStep")
		c1 = right.column()
		c1.scale_y = 2
		c1.operator("BBOperatorExport")
		c1.operator("BBOperatorAnim")
		num = ((scene.frame_end - scene.frame_start)/scene.BBExportStep)+1
		c1.label("A total of %d frames will be exported." % num)
