#####################################################################
# Contains all the core procedure for BioBlender
#####################################################################

import time, os, pickle, shutil, math
from bpy import *
from . import store
from .settings import *
from .parsers import *
from .helpers import *
from .web import *
from .writer_list import *
from mathutils import *
# from .sim import *

# print some debug info
def info():
	print ("#"*60)
	if linux:
		print("DEBUG: Using Linux file separator")
	else:
		print("DEBUG: Using Windows file separator")
	print ("DEBUG: Script Home is %s" % homePath)
	if os.path.exists(pyMolPath):
		print ("DEBUG: PyMol found at %s" % pyMolPath)
	else:
		print ("ERROR: PyMol NOT found.")

	if os.path.exists(pyPath):
		print ("DEBUG: Python 2 found at %s" % pyPath)
		if os.path.exists(pyPath[:-10]+"/Lib/site-packages/numpy"):
			print ("DEBUG: NumPy found")
		else:
			print ("ERROR: NumPy NOT found.")
	else:
		print ("ERROR: Python 2 and NumPy NOT found")

	print ("#"*60)

def index_element(this_list, elemen_list):
	for ik in range(len(this_list)):
		if elemen_list==this_list[ik]:
			return ik

# Save the session variables to disk
def sessionSave(verbose =  False):
	# if the blender file is not saved yet, do nothing
	if not data.is_dirty:
		# Serialize the the data and save to disk
		with open(data.filepath + ".cache","wb") as filedump:
			pickle.dump(store.modelContainer, filedump)
		if verbose: print ("Persistent Session Saved")
	else:
		if verbose: print ("Warning: Blender file needs to be saved first to create persistent session data")


# Load the saved variables from disk
def sessionLoad(verbose =  False):
	# if the blender file is not saved yet, do nothing
	# if store.modelContainer is empty but there is already a 'large' number of object in the scene:
	if not data.is_dirty and len(data.objects) > 500:
		# try to load serialized data from disk or fail silently
		try:
			with open(data.filepath + ".cache","rb") as filedump:
				store.modelContainer = pickle.load(filedump)
			select(data.objects[90].name)    # to select 'something' in the scene
			for model in store.modelContainer:
				store.activeTag = model
			if verbose: print ("Persistent session loaded")
		except Exception as E:
			print ("Warning: Error when loading session cache:", E)


#Not used, for normal mode analysis using vector force (only in BGE)
def importVectorPDB(): 
	print ("computeNMA")		
	
#Not used, for normal mode analysis using vector force (only in BGE)
def computeNormalModeVectors() : 
	print ("computeNMA")
	scene = context.scene
	name=scene.BBModelRemark
	inputpath="\""  + absPath(scene.BBImportPath) + "\""
	print(inputpath)
	if os.path.isfile(inputpath) : 
		modestr = scene.BBNormalModeAnalysis
		mode = int(modestr)+1
		struct = "--all"
		name=scene.BBModelRemark
		rmsd=context.scene.BBNMARMSD
		gamma = context.scene.BBNMAGamma
		cutoff = context.scene.BBNMACutoff		 
		nbconfiguration=context.scene.BBNMANbModel
		outputpath=homePath+"fetched/"+name+"_"+"Mode"+str(mode)+"_vectors"+".pdb"
		command = "%s %s -i %s -o %s -m %d" % (pyPath, homePath+"bin/nma/nmavectors.py", inputpath, outputpath, mode)

		print("Command:")
		print(str(command))
		p = launch(exeName=command, async = False)
		scene.BBImportPath=outputpath
		importPreview()
	else : 
		print ("File does not exist !!")


def computeNormalModeTrajectories() : 
	print ("computeNMA")
	scene = context.scene
	name=scene.BBModelRemark
	inputpath=absPath(scene.BBImportPath)
	
	print("scene.BBImportPath")
	print(scene.BBImportPath)
	print("inputpath")
	print(inputpath)
	if os.path.isfile(inputpath) : 
		modestr = scene.BBNormalModeAnalysis
		mode = int(modestr)+1
		struct = "--all"
		name=scene.BBModelRemark
		rmsd=context.scene.BBNMARMSD
		gamma = context.scene.BBNMAGamma
		cutoff = context.scene.BBNMACutoff		 
		nbconfiguration=context.scene.BBNMANbModel
		outputpath=homePath+"fetched/"+name+"_"+"Mode"+str(mode)+"_"+struct+"_"+str(rmsd)+"_"+str(nbconfiguration)+".pdb"
		print("outputpath")
		print(outputpath)
		command = "%s %s -i %s -o %s -m %d -r %f -n %d %s " % (pyPath, homePath+"bin/nma/nma.py", "\""+inputpath+"\"", outputpath, mode, rmsd, nbconfiguration, struct)

		print("Command:")
		print(str(command))
		p = launch(exeName=command, async = False)
		scene.BBImportPath=outputpath
		importPreview()
	else : 
		print ("File does not exist !!")


# Scan the file to be imported quickly to generate BBImportOrder list
def importPreview(verbose =  False):
	# aliases
	scene = context.scene
	filePath = absPath(scene.BBImportPath)

	# get PDB straight from PDB.org
	if len(filePath)==4:
		retrivedFile = pdbdotorg(filePath)
		if retrivedFile:
			scene.BBImportFeedback = "Found matching Protein on PDB.org"
			scene.BBImportPath = retrivedFile
		else:
			scene.BBImportFeedback = "Nothing matching this ID found on PDB.org"
			return False


	extension = filePath.lower().endswith
	try:
		file = open(filePath,"r")
	except:
		scene.BBImportFeedback = "File not found"
		return False
	else:
		# open is successful, check file extension
		if extension(".pdb") or extension(".pqr"):
			modelCount = 0
			importFileModel = []
			
			
			# read the chain id
			importChainID = []
			
			# read file
			for line in file:
				line = PDBString(line)
				# count number of models in file using the start tag MODEL
				if line.get("tag") == "MODEL":
					currentModelID = line.get("modelID")
					importFileModel.append(currentModelID)
				# count number of models in file using the end tag MODEL
				elif line.get("tag") == "ENDMDL":
					modelCount += 1
					
				# count number of chains in file using the start tag ATOM    1
				if line.get("tag") == "ATOM":
					currentChainID = line.get("chainID")
					chainCount = 0
					for elem in importChainID:
						if elem == currentChainID:
							chainCount += 1
					if chainCount == 0:
						importChainID.append(currentChainID)

			# Special case for files containing 1 model with no opening model tag
			if len(importFileModel) == 0: importFileModel.append(0)

			# show list of models for importer to load
			scene.BBImportOrder = str(importFileModel)[1:-1]

			# if all okay, display feedback message
			scene.BBImportFeedback = "File contains " + str(len(importFileModel)) + " PDB Models"
			
			# if all okay, display feedback message
			scene.BBImportChain = "File contains chains " + str(importChainID)
			file.close()
			return True

		if extension(".txt") or extension(".csv"):
			# simply get list of all files from a th specified directory
			fileList = os.listdir(os.path.dirname(filePath))
			fileListClean = []
			for f in fileList:
				# find only matching extensions
				ext = f[-4:].lower()
				if  ext == '.txt' or ext == '.csv':
					fileListClean.append (f)
			scene.BBImportOrder = ", ".join(fileListClean)
			# if all okay, display feedback message
			scene.BBImportFeedback = "Folder contains " + str(len(fileListClean)) + " text Files"
			file.close()
			return True

		# otherwise...
		scene.BBImportFeedback = "Error: Unrecognized File Extension"
		file.close()
		return False


# Load the file
def importFile(verbose = False):
	modelprog=[]
	start = time.time()
	#print("importFile")
	# Start with a clean slate
	modelList = []
	mainChainCache = []		# a cache to that contains only mainchain atoms
	mainChainCache_Nucleic = []
	mainChainCache_Nucleic_Filtered=[]
	chainCache = {}			# a cache to that contains all non-H atoms
	chainCache_Nucleic = {}
	model = {}
	currentModelID = 0
	curFrame = 1
	ops.object.select_all(action="DESELECT")
	context.scene.objects.active = None

	# alias
	scene = context.scene
	filePath = absPath(scene.BBImportPath)
	id = scene.BBModelRemark
	extension = filePath.lower().endswith

	# check to avoid import models with same name
	if id in store.modelContainer:
		raise Exception("Error: Duplicate Name Tag.  Try importing again with a different name.")

	# user input sanity check
	if getNumModel() == -1:
		raise Exception("Error: Invalid user ordering of model sequence.  Use comma to separate values")

	# Parse PDB
	if extension(".pdb") or extension(".pqr"):
		modelOrder = [int(model) for model in scene.BBImportOrder.split(',')]
		
		# open file (assuming input is valid)
		with open(filePath,"r") as file:
			for line in file:
				line = line.replace("\n", "")
				line = line.replace("\r", "")
				line = PDBString(line)

				tag = line.get("tag")

				# if tag is MODEL, load model ID
				if tag == "MODEL":
					currentModelID = line.get("modelID")

				# if tag is ATOM, load column data (skip this if currentModelID is not in list of models)
				elif (currentModelID in modelOrder) and (tag == "ATOM" or tag == "HETATM"):

					# check for element type
					atomName = line.get("name")
					elementName = line.get("element")
					elementTypeResidue=line.get("aminoName").strip()
					atomtype=line.get("tag")

					# skip water
					if line.get("aminoName") == "HOH":
						continue

					# decide if hydrogen should be skipped
					if not scene.BBImportHydrogen and elementName == H:
						continue

					# decide if sidechain should be skipped
					#if not scene.BBSideChain and not (atomName==N or atomName==C or (atomName==CA and elementName!=CA)):
					#	continue

					# build name string and add all atom data to dictionary
					key = id + "#" + line.get("serial").rjust(5, "0")
					model[key] = line

					# add mchain atom data to dictionary for building bonds
					if atomName==N or atomName==C or (atomName==CA and elementName!=CA):
						if key not in mainChainCache: mainChainCache.append(key)
					if atomName in store.NucleicAtoms:
						if atomName in store.NucleicAtoms_Filtered:
							if key not in mainChainCache_Nucleic_Filtered: mainChainCache_Nucleic_Filtered.append(key)
						else:
							if key not in mainChainCache_Nucleic: mainChainCache_Nucleic.append(key)
						if atomName =="C3'": mainChainCache_Nucleic.append(key)
					# add all atom data to dictionary for building bonds
					elementTypeNucleic=["D","A","U","G","C","DC","DG","DA","DT"]
					if elementName != H and (not elementTypeResidue in elementTypeNucleic ) and atomtype=="ATOM":
						chainCache[key] = line.get("aminoName") + "#" + line.get("chainSeq") + "#" + line.get("name") + "#" + line.get("chainID") + "#" + line.get("element")
					if elementName != H and (elementTypeResidue in elementTypeNucleic) and atomtype=="ATOM":
						chainCache_Nucleic[key] = line.get("aminoName") + "#" + line.get("chainSeq") + "#" + line.get("name") + "#" + line.get("chainID") + "#" + line.get("element")

				# when the end of a model is reached, add the model dictionary to the list
				
				if tag == "ENDMDL" or tag=="END" or tag == "MODEL" and (currentModelID in modelOrder):
					if model:
						modelList.append(model)
						modelprog.append(index_element(this_list=modelOrder,elemen_list=currentModelID))
						model = {}



	# Parse text files sequence
	if extension(".txt") or extension(".csv"):
		modelOrder = scene.BBImportOrder.split(',')
		for fileName in modelOrder:
			# open each file from the list
			with open(os.path.dirname(filePath)+"/"+fileName.strip(),"r") as f:
				atomCounter = 0
				# force the atom type
				char = C

				# for each line of the file
				for line in f:
					key = id + "#" + str(atomCounter).rjust(5, "0")
					line = line.replace('"', "")	# cleanup some unconforming data
					split_line = line.split()
					x = "%8.3f" % float(split_line[0])
					y = "%8.3f" % float(split_line[1])
					z = "%8.3f" % float(split_line[2])
					# a somewhat hackish way to generate a PDB-conformant string for the unified reader to use
					line = PDBString("ATOM                                                                         " + char)
					# insert location in-situ
					line = line.set(30,x)
					line = line.set(38,y)
					line = line.set(46,z)

					model[key] = line
					atomCounter += 1
				modelList.append(model)
				model = {}
	# #trying to sort modelList
	try:
		modelList1=[]
		for ikt in modelList:
			modelList1.append(ikt)
		for ikt in range(len(modelprog)):
			modelList1[int(modelprog[ikt])]=modelList[ikt]
		modelList=[]
		for ikt in modelList1:
			modelList.append(ikt)
	except Exception as E:
		raise Exception ("Error sorting model list ", E)

	try:
		if scene.BBSpaceFill:
			objName = "atomFill"
		else:
			objName = "atom"


		# Append atom object from external file and create atom object for each element
		#if linux:
		#	path = homePath+"data/library.blend/Object/"
		#else:
		#	path = homePath+"data\\library.blend\\Object\\"

		path= homePath+"data"+os.sep+"library.blend"+os.sep+"Object"+os.sep
		
		
		ops.wm.link_append(filepath = path + objName, filename = objName, directory = path, link = False)
		data.objects[objName].name ="atom"
		context.scene.objects.active = data.objects["atom"]
		context.active_object.game.use_collision_bounds = False
		# import object to active layer, for multi-import support:
		context.active_object.layers = context.scene.layers
	except Exception as E:
		raise Exception ("Template atom object cannot be loaded from library: ", E)



	# Make high res atom model
	if scene.BBHighQuality:
		ops.object.editmode_toggle()
		ops.mesh.subdivide(number_cuts=1, smoothness=1)
		ops.object.editmode_toggle()


	# generate materials on first run
	if store.modelContainer == {}:
		data.materials['atom'].name = C
		data.materials['C'].diffuse_color = color[C]

		data.materials['C'].copy()
		data.materials['C.001'].name = CA
		data.materials['CA'].diffuse_color =  color[CA]

		data.materials['CA'].copy()
		data.materials['CA.001'].name = H
		data.materials['H'].diffuse_color = color[H]

		data.materials['H'].copy()
		data.materials['H.001'].name = N
		data.materials['N'].diffuse_color = color[N]

		data.materials['N'].copy()
		data.materials['N.001'].name = O
		data.materials['O'].diffuse_color = color[O]

		data.materials['O'].copy()
		data.materials['O.001'].name = S
		data.materials['S'].diffuse_color =  color[S]
		
		data.materials['S'].copy()
		data.materials['S.001'].name = ZN
		data.materials['ZN'].diffuse_color =  color[ZN]
		
		data.materials['ZN'].copy()
		data.materials['ZN.001'].name = P
		data.materials['P'].diffuse_color =  color[P]
		
		data.materials['P'].copy()
		data.materials['P.001'].name = FE
		data.materials['FE'].diffuse_color =  color[FE]
		
		data.materials['FE'].copy()
		data.materials['FE.001'].name = MG
		data.materials['MG'].diffuse_color =  color[MG]
		
		data.materials['MG'].copy()
		data.materials['MG.001'].name = MN
		data.materials['MN'].diffuse_color =  color[MN]
		
		data.materials['MN'].copy()
		data.materials['MN.001'].name = CU
		data.materials['CU'].diffuse_color =  color[CU]
		
		data.materials['CU'].copy()
		data.materials['CU.001'].name = NA
		data.materials['NA'].diffuse_color =  color[NA]
		
		data.materials['NA'].copy()
		data.materials['NA.001'].name = K
		data.materials['K'].diffuse_color =  color[K]
		
		data.materials['K'].copy()
		data.materials['K.001'].name = CL
		data.materials['CL'].diffuse_color =  color[CL]
		
		data.materials['CL'].copy()
		data.materials['CL.001'].name = F
		data.materials['F'].diffuse_color =  color[F]
		
		create_fi_materials()
		
	# Build 3D scene from modelList
	for model in modelList:
		# reset
		ops.object.select_all(action="DESELECT")
		scene.objects.active = None
		scene.frame_set(curFrame)

		# on first model, Place atoms in scene
		if curFrame == 1:
			modelCopy = model.copy()

			# select template atom
			data.objects["atom"].hide = False
			data.objects["atom"].select = True

			# temporary rename template atom
			data.objects["atom"].name = id

			try:
				# (count - 1) because there is the original template object.
				print ("Making " +str(len(model))+" objects")
				ops.object.duplicate(linked = not scene.BBSingleUser, mode=1, copies = len(model)-1)
			except:
				# We are using a specially patched version of the duplicate operator that is faster
				print ("Warning: Fast duplicate instancing is not supported by this version of Blender. Trying Brute Force approach.")
				for i in range(len(model)-1):
					ops.object.duplicate(linked = not scene.BBSingleUser, mode=1)

			try:
				# walk through list of objects and set name, location and material for each atom
				for i, obj in enumerate(data.objects):
					# if the object is the one of those we just created (i.e. if name matches xx.0000)
					if obj.name.split(".")[0] == id:
						# descructively walk through the modelCopy varible
						entry = modelCopy.popitem()
						# assign obj name, material, etc.  object Locations are assigned later
						obj.name = entry[0]
						index = entry[1].get("element")
						obj.material_slots[0].material = data.materials[index]
						# adjust radius
						if scene.BBSpaceFill:
							obj.scale = [scale_vdw[index][0], scale_vdw[index][0], scale_vdw[index][0]]
							obj.game.radius = scale_vdw[index][1]
						else:
							obj.scale = [scale_cov[index][0], scale_cov[index][0], scale_cov[index][0]]
							obj.game.radius = scale_cov[index][1]						
						# add atom info as RNA string to each object
						obj.BBInfo = entry[1][:]
				#scene.BBChainImported=Chain_atoms
				print("usciti dal try dopo il warning")
			except Exception as E:
				raise Exception ("Unable to generate 3D model from PDB File", E)

			if verbose: print ("Build time: " + str(time.time() - start))


			if scene.BBMakeBonds:
				print("making bonds now")
				try:
					cacheSize = len(mainChainCache) - 1
					print ("Adding " + str(cacheSize) + " mainchain Bonds")
					for i, entry in enumerate(mainChainCache):
						# Skipping Last Atom to avoid cyclic dependency
						if i < cacheSize:
							# Adding constraints, using atom position to correctly orient hinge x axis
							obj = data.objects[entry]
							# simply link the current object with the next object in the mainchain
							nextEntry = data.objects[mainChainCache[i+1]]
							#We have to know the location of 2 atoms before creating rigid body
							#obj.constraints["RigidBody Joint"].target = nextEntry
							line=modelList[0][entry]
							#print (line)
							obj.location=line.get("loc")
							line=modelList[0][mainChainCache[i+1]]
							#print (line)
							nextEntry.location=line.get("loc")
							
							addRigidBodyRotamer(obj,nextEntry)							
					#bonds for Nucleic Acids
					cacheSize = len(mainChainCache_Nucleic_Filtered) - 1
					print ("Adding " + str(cacheSize) + " mainchain_Nucleic Filtered Bonds")
					for i, entry in enumerate(mainChainCache_Nucleic_Filtered):
						# Skipping Last Atom to avoid cyclic dependency
						if i < cacheSize:
							# Adding constraints, using atom position to correctly orient hinge x axis
							obj = data.objects[entry]
							# simply link the current object with the next object in the mainchain
							nextEntry = data.objects[mainChainCache_Nucleic_Filtered[i+1]]
							#We have to know the location of 2 atoms before creating rigid body
							#obj.constraints["RigidBody Joint"].target = nextEntry
							line=modelList[0][entry]
							#print (line)
							obj.location=line.get("loc")
							line=modelList[0][mainChainCache_Nucleic_Filtered[i+1]]
							#print (line)
							nextEntry.location=line.get("loc")
							addRigidBodyRotamer(obj,nextEntry)							
							
							
					if verbose: print ("Link time: " + str(time.time() - start))

					chainCache = sorted(chainCache.items())
					print ("Adding " + str(len(chainCache)) + " sidechain Bonds")

					for entry in chainCache:
						line = entry[1].split("#")
						amac = line[0]
						chainSeq = line[1]
						atom = line[2]
						chainID = line[3]

						# skip mainchain atoms
						if (atom != C and atom != CA and atom != N and atom != H):
							# for side chain, look up parents based on rules
							parent = bondLookUp(atom=atom, amac=amac)

							# generate name of parents
							target = amac+ "#" + chainSeq + "#" + parent[0] + "#" + chainID + "#" + parent[1]

							# lookup name of blenderobject based on parent name
							targetKey = "atom"
							for item in chainCache:
								if item[1] == target:
									targetKey = item[0]
									break
							# set up the constraint
							obj = data.objects[entry[0]]
							# Adding constraints, using atom position to correctly orient hinge x axis
							#obj.constraints["RigidBody Joint"].target = data.objects[targetKey]
							line=modelList[0][entry[0]]
							#print (line)
							obj.location=line.get("loc")
							line=modelList[0][targetKey]
							#print (line)
							nextEntry.location=line.get("loc")

							addRigidBodyRotamer(obj,data.objects[targetKey])							

							
					chainCache = sorted(chainCache_Nucleic.items())
					print ("Adding " + str(len(chainCache)) + " sidechain Nucleic Bonds")

					for entry in chainCache:
						line = entry[1].split("#")
						amac = line[0]
						chainSeq = line[1]
						atom = line[2]
						chainID = line[3]

						# skip mainchain atoms
						#if atom=="O2P" or atom=="O1P" or atom=="C3'" or atom=="C2'"or atom=="C1'" or atom=="O4'":
						if not atom in store.NucleicAtoms:
							# for side chain, look up parents based on rules
							print("ATOM, AMAC:" + str(atom) + " " + str(amac))
							parent = bondLookUp_NucleicMain(atom=atom, amac=amac)
							print("parent: "+ str(parent))
							# generate name of parents
							target = amac+ "#" + chainSeq + "#" + parent[0] + "#" + chainID + "#" + parent[1]
							print("target " + str(target))
							# lookup name of blenderobject based on parent name
							targetKey = "atom"
							for item in chainCache:
								if item[1] == target:
									targetKey = item[0]
									break
							# set up the constraint
							print("entry0: " +  str(entry[0]))
							obj = data.objects[entry[0]]
							print("objfrom: " +  obj.name +" targetobject: "+ str(targetKey) + str(data.objects[targetKey]))
							# Adding constraints, using atom position to correctly orient hinge x axis
							#obj.constraints["RigidBody Joint"].target = data.objects[targetKey]
							line=modelList[0][entry[0]]
							#print (line)
							obj.location=line.get("loc")
							line=modelList[0][targetKey]
							#print (line)							
							nextEntry.location=line.get("loc")
							addRigidBodyRotamer(obj,data.objects[targetKey])							
							
				except Exception as E:
					raise Exception ("Unable to generate all bonds and constraints:", E)
				print("Finished making bonds!")
				if verbose: print ("Link time: " + str(time.time() - start))

		try:
			# for all models, insert key frame
			print("taking OBJ location")
			for key, line in model.items():
				OBJ = data.objects[key]
				OBJ.select = True
				OBJ.location = line.get("loc")
			print("OBJ location taken")
		except Exception as E:
				raise Exception ("Unable to place 3D atoms:", E)
				
		if len(modelList) != 1:
			# insert keyframe for animations
			print("inserting kframe?")
			ops.anim.keyframe_insert_menu(type=-1)

			# advance frame
			print ("Frame: " + str(curFrame))
			scene.frame_end = curFrame
			curFrame += scene.BBDeltaFrame

	if verbose: print ("Model animation time: " + str(time.time() - start))

	# Cleanup
	scene.frame_set(1)
	ops.object.select_all(action="DESELECT")
	store.activeTag = id

	# move camera target to center of molecule
	ob = select("Empty")
	ob.location = atomCentre()
	print("ok empty!")

	# play animation
	if len(modelList) > 1 and not context.screen.is_animation_playing:
		ops.screen.animation_play()

	# Finished!  Finally add model to dictionary for multi-model import support
	# for multi-conformation models, we only add the first model to the store.modelContainer
	store.modelContainer[id] = modelList[0]

	sessionSave()
	print ("Finished Importing")
	#grainMain()

# depending on view mode, selectively hide certain object based on atom definition
def updateView(residue = None, verbose = False):

	scene = context.scene
	viewMode = scene.BBViewFilter

	# select amino acid by group
	if residue:
		# skip none atomic object
		if not residue.BBInfo:
			return

		#ops.object.select_all(action="DESELECT")
		# get residue ID
		seq = PDBString(residue.BBInfo).get("chainSeq")
		id = PDBString(residue.BBInfo).get("chainID")
		# get the model from the model container
		selectedModel = store.modelContainer[context.active_object.name.split("#")[0]]
		for objName, BBInfo in selectedModel.items():
			if BBInfo.get("chainSeq") == seq and BBInfo.get("chainID") == id:
				data.objects[objName].select = True

				
		return

	# try to select surface
	obj = select("Surface")
	if viewMode == "4":
		data.worlds[0].light_settings.use_environment_light = False
		if not obj:
		# generate surface if does not exist
			setup()
			print("first setup made")
			surface()
			print("surface made")
		elif obj.hide:
		# unhide surface if it's hidden
			obj.hide = False
			obj.hide_render = False
	else:
		data.worlds[0].light_settings.use_environment_light = True
		if obj:
			# hide surface if already exist
			obj.hide = True
			obj.hide_render = True


	# if store.modelContainer and store.activeTag:
		# model = store.modelContainer[store.activeTag]
		# for key, line in model.items():
	for obj in data.objects:
		#obj = data.objects[key]
		obj.hide = False
		obj.hide_render = False
		obj.draw_type = "TEXTURED"
		if re.search("#",obj.name):
			line=obj.BBInfo
			line = PDBString(line)
			elementName = line.get("element")
			atomName = line.get("name")

			# hide all
			if viewMode == "0":
				obj.hide = True
				obj.hide_render = True
			# Main Chain Only
			elif viewMode == "1":
				if not (atomName == N or atomName == C or (atomName == CA and elementName != CA) or (atomName in store.NucleicAtoms) or (atomName in store.NucleicAtoms_Filtered)):
					obj.hide = True
					obj.hide_render = True
			# Main Chain and Side Chain Only
			elif viewMode == '2' and elementName ==  H:
				obj.hide = True
				obj.hide_render = True
			# Main Chain and Side Chain Only and H, everything.
			elif viewMode == '4':
				obj.hide = True
				obj.hide_render = True



# start the BGE
def startGame(verbose =  False):
	if context.scene.BBSolveMotion == 0:
		context.scene.game_settings.physics_engine = "NONE"
	if context.scene.BBSolveMotion > 0:
		context.scene.game_settings.physics_engine = "BULLET"
	if context.vertex_paint_object:
		ops.paint.vertex_paint_toggle()
	for i, obj in enumerate(data.objects):
		obj.game.radius=context.scene.BBDeltaPhysicRadius
	setViewShading("TEXTURED")
	ops.view3d.game_start()
	setViewShading("SOLID")


# export scene to PDB file
# if no path is specified, it writes to tmp.pdb
def exportPDB(path = homePath+"tmp/tmp.pdb", tag = None, verbose = False):
	print("exporting PDB")
	if verbose: print ("Exporting model '%s' to %s" %(tag, path))
	scene = context.scene
	outPath = absPath(path)
	model = store.modelContainer[tag]
	ordered = sorted(model.keys())

	with open(outPath, "w") as outFile:
		for obID in ordered:
			try:
				loc = data.objects[obID].location
				info = data.objects[obID].BBInfo
			except Exception as E:
				raise Exception ("Error: 3D scene and internal dictionary does not match: ", E)

			x = "%8.3f" % loc[0]
			y = "%8.3f" % loc[1]
			z = "%8.3f" % loc[2]

			# convert line to pdbstring class
			line = PDBString(info)
			# clear location column
			line = line.set(30,"                         ")
			# insert new location
			line = line.set(30,x)
			line = line.set(38,y)
			line = line.set(46,z)

			outFile.write (line+"\n")

		outFile.write("ENDMDL")



def setup(verbose = False, clear = True):
	scene = context.scene
	PDBPathBBImp=scene.BBImportPath#[12:len(scene.BBImportPath)]
	#PDBPathBBImp="//2L2J.pdb"
	print("PDBAthimport"+PDBPathBBImp)
	
	PDBPath = absPath(PDBPathBBImp)
	print("PDBAth"+PDBPath)
	if clear:
		print ("===")
		if os.path.isdir(homePath+"tmp"):
			print("there's a tmp in the homepath!")
			print(homePath+"tmp")
			#os.chdir(homePath+"tmp")
			shutil.rmtree(homePath+"tmp")
			os.mkdir(homePath+"tmp")
		else:
			os.mkdir(homePath+"tmp")

	if verbose: print ("Copying original PDB to tmp directory")
	print("I'm trying to copy"+PDBPath)
	shutil.copy(PDBPath, homePath+"tmp/original.pdb")

	if store.modelContainer:
		exportPDB(tag = store.activeTag)
	else:
		print ("Warning: Using original PDB, not the one generated by Blender")
		shutil.copy(homePath+"tmp/original.pdb", homePath+"tmp/tmp.pdb")

	# clear zero-user datablocks
	if True:
		for datablock in data.meshes:
			if datablock.users == 0:
				data.meshes.remove(datablock)
		for datablock in data.curves:
				if datablock.users == 0:
					data.curves.remove(datablock)
		for datablock in data.objects:
			if datablock.users == 0:
				data.objects.remove(datablock)
	print("Setup is complete!")

# Import the surface generated from PyMol
def surface(verbose = True):
	scene = context.scene
	res = scene.BBMLPSolventRadius
	print("into surface")
	if select("Surface"):
		ops.object.delete()

	if scene.BBHighQuality:
		quality = "1"
	else:
		quality = "0"

	if verbose: print ("Generating PyMol script")
	with open(homePath+"tmp/surface.pml", mode="w") as f:
		f.write("# This file is automatically generated by BioBlender at runtime.\n")
		f.write("# Modifying it manually might not have an effect.\n")
		f.write("load "+  homePath+"tmp/tmp.pdb\n")
		f.write('cmd.hide("lines"  ,"tmp")\n')
		f.write('cmd.set("surface_quality"  ,"%s")\n' % quality)
		f.write('cmd.show("surface","tmp")\n')
		f.write('set solvent_radius,'+ str(res) +'\n')
		f.write('cmd.reset()\n')
		f.write('cmd.origin(position=[0,0,0])\n')
		f.write('cmd.center("origin")\n')
		f.write("cmd.move('z',-cmd.get_view()[11])\n")
		f.write("save " + homePath+"tmp"+os.sep+"tmp.wrl\n")
		f.write("quit")


	print ("Making Surface using PyMOL")
	command = "%s +2 -c -u %s" % (pyMolPath, homePath+"tmp/surface.pml")
	launch(exeName=command)

	print ("Importing WRL surface geometry into Blender")
	uriSource = homePath+"data/loader.blend"
	uriDest = homePath+"tmp/loader.blend"
	shutil.copy(uriSource, uriDest)
	command = "%s %s --garbage" % (blenderPath, homePath+"tmp/loader.blend")
	launch(exeName=command)

	#Append surface object from external file (doubles removed, UV unwrapped, cleaned up)
	#this is MUCH faster than doing UV unwrapping in the current scene
	# if linux:
		# path = homePath+"tmp/loader.blend/Object/"
	# else:
		# path = homePath+"tmp\\loader.blend\\Object\\"
		
	path = homePath+"tmp"+os.sep+"loader.blend"+os.sep+"Object"+os.sep
	
		
	ops.wm.link_append(filepath = path + "Surface", filename = "Surface", directory = path, link = False)



# do MLP visualization
def mlp(force, verbose = True):
	scene = context.scene
	formula = scene.BBMLPFormula
	spacing = scene.BBMLPGridSpacing

	# input: raw vertex ID
	# output: 3f color
	def getVar(rawID):
		try:
			val = store.dxCache[rawID]
		except:
			v = ob.data.vertices[rawID].co
			dimx = store.dimension[0]
			dimy = store.dimension[1]
			dimz = store.dimension[2]
			deltax = store.delta[0]
			deltay = store.delta[1]
			deltaz = store.delta[2]
			cellx = int((v[0] - store.origin[0]) / deltax)
			celly = int((v[1] - store.origin[1]) / deltay)
			cellz = int((v[2] - store.origin[2]) / deltaz)
			mmm = store.dxData[cellz + ((celly)*dimz) + ((cellx)*dimz*dimy)]
			pmm = store.dxData[cellz + ((celly)*dimz) + ((cellx+1)*dimz*dimy)]
			mpm = store.dxData[cellz + ((celly+1)*dimz) + ((cellx)*dimz*dimy)]
			mmp = store.dxData[cellz+1 + ((celly)*dimz) + ((cellx)*dimz*dimy)]
			ppm = store.dxData[cellz + ((celly+1)*dimz) + ((cellx+1)*dimz*dimy)]
			mpp = store.dxData[cellz+1 + ((celly+1)*dimz) + ((cellx)*dimz*dimy)]
			pmp = store.dxData[cellz+1 + ((celly)*dimz) + ((cellx+1)*dimz*dimy)]
			ppp = store.dxData[cellz+1 + ((celly+1)*dimz) + ((cellx+1)*dimz*dimy)]
			wxp = 1.0-(math.fabs(v[0] - (store.origin[0] + (deltax*(cellx+1)))))/deltax
			wxm = 1.0-(math.fabs(v[0] - (store.origin[0] + (deltax*(cellx)))))/deltax
			wyp = 1.0-(math.fabs(v[1] - (store.origin[1] + (deltay*(celly+1)))))/deltay
			wym = 1.0-(math.fabs(v[1] - (store.origin[1] + (deltay*(celly)))))/deltay
			wzp = 1.0-(math.fabs(v[2] - (store.origin[2] + (deltaz*(cellz+1)))))/deltaz
			wzm = 1.0-(math.fabs(v[2] - (store.origin[2] + (deltaz*(cellz)))))/deltaz
			onz_xmym = (wzp*mmp)+(wzm*mmm)
			onz_xpym = (wzp*pmp)+(wzm*pmm)
			onz_xmyp = (wzp*mpp)+(wzm*mpm)
			onz_xpyp = (wzp*ppp)+(wzm*ppm)
			onx_yp = (wxp*onz_xpyp)+(wxm*onz_xmyp)
			onx_ym = (wxp*onz_xpym)+(wxm*onz_xmym)
			val = (wyp*onx_yp)+(wym*onx_ym)
			store.dxCache[rawID] = val

		# map values
		if(val >= 0.0):
			val=(val+1.0)/2.0*scene.BBMLPContrast+scene.BBMLPBrightness
		else:
			val=(val+3.0)/6.0*scene.BBMLPContrast+scene.BBMLPBrightness

		return [val,val,val]




	if force:

		setup()

		# select formula for PyMLP script
		if formula == "0": method = "dubost"
		elif formula == "1": method = "testa"
		elif formula == "2": method = "fauchere"
		elif formula == "3": method = "brasseur"
		elif formula == "4": method = "buckingham"

		# Launch this in a separate process
		print ("Running PyMLP")
		command = "%s %s -i %s -m %s -s %f -v" % (pyPath, homePath+"bin/pyMLP-1.0/pyMLP.py", homePath+"tmp/tmp.pdb", method, spacing)
		print("Command:")
		print(str(command))
		p = launch(exeName=command, async = True)
		print("command succeded")
		# do it in parallel with pyMLP
		surface()

		# wait for pyMLP to finish
		wait(p)

		# purge the all old data
		store.dxCache = {}
		store.dxData = []			# list[n] of Potential data
		store.dimension = []		# list[3] of dx grid store.dimension
		store.origin = []			# list[3] of dx grid store.origin
		store.delta = []			# list[3] of dx grid store.increment

		print ("Loading MLP values into Blender")
		with open(homePath+"tmp"+os.sep+"tmp.dx") as dx:
			for line in dx:
				# skip comments starting with #
				if line[0] == "#": continue
				if not store.dimension:
					#get the store.dimension and convert to integer
					dim = line.split()[-3:]
					store.dimension = [int(d) for d in dim]
					size = store.dimension[0]*store.dimension[1]*store.dimension[2]
					continue

				if not store.origin:
					# get the store.origin
					org = line.split()[-3:]
					store.origin = [float(o) for o in org]
					continue
					
				if not store.delta:
					# get the increment delta
					x = float(line.split()[-3:][0])
					line = dx.readline()
					y = float(line.split()[-3:][1])
					line = dx.readline()
					z = float(line.split()[-3:][2])
					store.delta = [x,y,z]

					# ignore more garbage lines
					dx.readline()
					dx.readline()
					continue

				# load as much data as we should, ignoring the rest of the file
				if (len(store.dxData) >= size):
					break

				# Load the data
				# Convert dx data from str to float, then save to list
				[store.dxData.append(float(coord)) for coord in line.split()]



	# quick and dirty update starts here
	if store.dxData:
		ob = select("Surface")
		if not context.vertex_paint_object:
			ops.paint.vertex_paint_toggle()

		# Calculate MLP surface color
		vColor = []
		for f in ob.data.faces:
			vColor.extend(getVar(f.vertices_raw[0]))
		ob.data.vertex_colors[0].data.foreach_set('color1', vColor)
		vColor = []
		for f in ob.data.faces:
			vColor.extend(getVar(f.vertices_raw[1]))
		ob.data.vertex_colors[0].data.foreach_set('color2', vColor)
		vColor = []
		for f in ob.data.faces:
			vColor.extend(getVar(f.vertices_raw[2]))
		ob.data.vertex_colors[0].data.foreach_set('color3', vColor)

		# needed to make sure VBO is up to date
		ob.data.update()
	print("mlp function is complete!!!!!")


def mlpRender(verbose =  True):
	scene = context.scene
	# Stop if no surface is found
	ob = select("Surface")
	if not ob:
		raise Exception("No MLP Surface Found, select surface view first")

	# Stop if no dx data is loaded
	if not store.dxData:
		raise Exception("No MLP data is loaded.  Run MLP calculation first")

	# create image data block
	try:
		firstTime = False
		image = data.images["MLPBaked.png"]
	except:
		firstTime = True
		image=data.images.new(name="MLPBaked", width=2048, height=2048)

	# set material
	if firstTime:
		mat = data.materials.new("matMLP")
		mat.use_shadeless=True
		mat.use_vertex_color_paint=True
		ob.data.materials.append(mat)
		
	else:
		mat = data.materials["matMLP"]
		mat.use_shadeless=True
		mat.use_vertex_color_paint=True
		if not ob.data.materials:
			ob.data.materials.append(mat)
			

	print ("Baking MLP textures")
	
	# save and bake
	image.source = "GENERATED"
	image.generated_height = 2048
	image.generated_width = 2048
	for uv in ob.data.uv_textures[0].data:
		uv.image=image
	scene.render.use_raytrace = False
	ops.object.bake_image()
	scene.render.use_raytrace = True
	os.chdir(homePath+"tmp/")
	#image.save()
	image.save_render(homePath+"tmp/MLPBaked.png")

	# copy the needed files
	uriSource = homePath+"data/noise.png"
	uriDest = homePath+"tmp/noise.png"
	shutil.copy(uriSource, uriDest)

	uriSource = homePath+"data/composite.blend"
	uriDest = homePath+"tmp/composite.blend"
	shutil.copy(uriSource, uriDest)

	# render out composite texture
	command = "%s -b %s -f 1" % (blenderPath, homePath+"tmp/composite.blend")
	launch(exeName=command)
	


	# set materials
	mat.specular_shader=("TOON")
	mat.specular_toon_size=0.2
	mat.specular_toon_smooth=0.0
	mat.specular_intensity=0.0
	mat.use_shadeless=False
	mat.use_vertex_color_paint=False
	mat.use_shadows=False

	# setup textures
	if firstTime:
		img_bump = data.images.load(homePath+"tmp/0001.png")
		tex_bump = data.textures.new('bump', type="IMAGE")
		tex_bump.image = img_bump
		#mat.add_texture(texture=tex_bump, texture_coords="UV", map_to="NORMAL")
		mtex = mat.texture_slots.add()
		mtex.texture = tex_bump
		mtex.texture_coords = 'UV'
		mtex.use_map_normal = True
		mtex.use_map_color_diffuse = False
		
		data.textures["bump"]
		mat.texture_slots[0].normal_factor=10
		img_baked = data.images.load(homePath+"tmp/MLPBaked.png")
		tex_spec=data.textures.new('specular', type="IMAGE")
		tex_spec.image = img_baked
		tex_spec.contrast = 4.0
		#mat.add_texture(texture=tex_spec, texture_coords="UV", map_to="SPECULARITY")
		mtex = mat.texture_slots.add()
		mtex.texture = tex_spec
		mtex.texture_coords = 'UV'
		mtex.use_map_color_diffuse = False
		mtex.use_map_specular = True
		mat.texture_slots[1].use_rgb_to_intensity=True
		mat.texture_slots[1].default_value=0.18

	# refresh all images
	for img in data.images:
		img.reload()




def ep(animation, verbose =  True):
	scene = context.scene

	setup(clear=False)

	# make new surface
	if not (animation or select("Surface")):
		print ("Generating Surface")
		surface()


	if not animation or (scene.frame_current % 5 == 1):
		print ("Generating EP Curves")

		# has H?
		#extraCommand = ""
		with open(homePath+"tmp/tmp.pdb","r") as file:
			for line in file:
				line = line.replace("\n", "")
				line = line.replace("\r", "")
				line = PDBString(line)
				tag = line.get("tag")

				# if tag is ATOM, load column data
				if (tag == "ATOM" or tag == "HETATM"):
					# check for element type
					if line.get("element") == H:
						extraCommand = "--assign-only"
						break


		# select the forcefield
		forcefield = scene.BBForceField
		if forcefield == "0": method = "amber"
		elif forcefield == "1": method = "charmm"
		elif forcefield == "2": method = "parse"
		elif forcefield == "3": method = "tyl06"
		elif forcefield == "4": method = "peoepb"
		elif forcefield == "5": method = "swanson"

		print ("Running PDB2PQR")
		os.chdir(homePath+"bin/pdb2pqr-1.6/")
		print(homePath)
		#command = "%s %s --apbs-input --ff=%s %s %s %s" % ("python", homePath+"bin/pdb2pqr-1.6/pdb2pqr.py", method, extraCommand, homePath+"tmp/tmp.pdb", homePath+"tmp/tmp.pqr")
		command = "%s %s --apbs-input --ff=%s %s %s" % ("python", homePath+"bin/pdb2pqr-1.6/pdb2pqr.py", method, homePath+"tmp/tmp.pdb", homePath+"tmp/tmp.pqr")
		
		print("pdb2pqr command:")
		print(command)
		launch(exeName=command)

		print ("Running inputgen.py")
		command = "%s %s --istrng=%f --method=auto --space=%f %s" % ("python", homePath+"bin/pdb2pqr-1.6/src/inputgen.py", scene.BBEPIonConc, scene.BBEPGridStep, homePath+"tmp/tmp.pqr")
		launch(exeName=command)

		print ("Running APBS")
		shutil.copy(homePath+"tmp/tmp.in", homePath+"bin/apbs-1.2.1/tmp.in")
		shutil.copy(homePath+"tmp/tmp.pqr", homePath+"bin/apbs-1.2.1/tmp.pqr")
		shutil.copy(homePath+"data/runAPBS.bat", homePath+"bin/apbs-1.2.1/runAPBS.bat")
		os.chdir(homePath+"bin/apbs-1.2.1/")
		command = homePath+"bin/apbs-1.2.1/runAPBS.bat"
		p = launch(exeName=command, async = True)

		print ("Saving obj")
		exportOBJ(homePath+"tmp/tmp")

		if len(store.epOBJ) >= maxCurveSet:
			# delete the oldest curve-sets out of the list.
			store.epOBJ.reverse()
			deletionList = store.epOBJ.pop()
			cleanEPObjs(deletionList)
			store.epOBJ.reverse()

		# sync
		wait(p)
		if os.path.isfile(homePath+"bin/apbs-1.2.1/pot-PE0.dx"):
			shutil.copy(homePath+"bin/apbs-1.2.1/pot-PE0.dx", homePath+"tmp/tmp.dx")
		else: 
			shutil.copy(homePath+"bin/apbs-1.2.1/pot.dx", homePath+"tmp/tmp.dx")


		print("Running Scivis")
		command = "%s %s %s %s %f %f %f %f %f" % (homePath+"bin/scivis/SCIVIS", homePath+"tmp/tmp.obj", homePath+"tmp/tmp.dx", homePath+"tmp/tmp.txt", scene.BBEPNumOfLine/10, scene.BBEPMinPot, 45, 1, 3)
		launch(exeName=command)

		print("Importing data into Blender")
		list = importEP(homePath+"tmp/tmp.txt")
		store.epOBJ.append(list)

		ob = select("Emitter")

		if animation:
			ob.particle_systems[0].settings.frame_start = scene.frame_current
			ob.particle_systems[0].settings.frame_end = scene.frame_current
			ob.particle_systems[0].settings.count = len(ob.data.vertices)
		else:
			if not context.screen.is_animation_playing:
				ops.screen.animation_play()






def exportSequence(verbose =  True, PDBOnly = False):
	scene = context.scene
	step = scene.BBExportStep
	start = scene.frame_start
	end = scene.frame_end
	renderMode = scene.BBRender
	a = time.time()

	if not store.modelContainer:
		raise Exception("No PDB file is loaded into Blender, cannot export anything.")

	pdbPath = os.path.normpath(absPath(scene.render.filepath))

	if not os.path.isdir(pdbPath):
		os.mkdir(absPath(pdbPath))

	i = start
	while i <= end:
		scene.frame_set(i)
		current_frame = scene.frame_current
		#current_frame = i
		
		if PDBOnly:
			# export PDB file only
			exportPDB(path = pdbPath+"/%s%d.pdb" % (store.activeTag, i), tag = store.activeTag, verbose = True)
		else:
			if renderMode == "0":
				#print ("Rendering atoms")
				scene.BBViewFilter = "3"
				updateView()
			elif renderMode == "1":
				#print ("Rendering Surface")
				setup()
				surface()
				scene.BBViewFilter = "4"
				updateView()
			elif renderMode == "2":
				#print ("Rendering MLP")
				mlp(force = True)
				mlpRender()
				scene.BBViewFilter = "4"
				updateView()
			elif renderMode == "3":
				#print ("Rendering EP")
				setup()
				surface()
				ep(animation=True)
				scene.BBViewFilter = "4"
				updateView()
				step = 1  # EP animation should not skip steps, due to particles behavior
			elif renderMode == "4":
				#print ("Rendering EP+MLP")
				mlp(force = True)
				mlpRender()
				ep(animation=True)
				scene.BBViewFilter = "4"
				updateView()
				step = 1   # EP animation should not skip steps, due to particles behavior

			# render frame
			renderSingle(i)

		i += step

	# clean up
	scene.frame_set(start)
	scene.frame_start = start
	scene.frame_end = end

	print (time.time()-a)

def session_write_aminoacids(verbose =  False):
	aaList = []
	Atoms_list=[]
	try:
		#when  proteins are imported, walk through list of objects  in the scene. If the object is a atom  belonging to the proteins, write aminoacid  chain, name, number and protein name in the list of aminoacids
		print(len(data.objects))
		for obj in context.scene.objects:
			if re.search("#",obj.name):
				splitname=obj.name.split("#")
				iteminfo=[splitname[0],PDBString(obj.BBInfo).get("chainID")+" "+"%5.5d" % int(PDBString(obj.BBInfo).get("chainSeq")) + " " +PDBString(obj.BBInfo).get("aminoName")]
				if not iteminfo in aaList:
					aaList.append(iteminfo)
				Atoms_list.append(obj.name.split("#")[0]+"#"+PDBString(obj.BBInfo).get("chainID")+"#"+obj.name.split("#")[1])
		aaList.sort()
		Atoms_list.sort()
		this_Chain="XY@"
		this_Protein="XY@"
		lastnumber=""
		ops.object.select_all(action="SELECT")
		ops.group.objects_remove()
		ops.object.select_all(action="DESELECT")
		for item in Atoms_list:
			if str(item.split("#")[1])!=this_Chain or str(item.split("#")[0])!=this_Protein:
				print("loading Chain_list")
				if this_Chain!="XY@":
					ops.group.create(name=lastnumber)
					ops.object.select_all(action="DESELECT")
					store.ChainModels[this_Protein+"#"+this_Chain]=lastnumber
				this_Chain=item.split("#")[1]
				this_Protein=item.split("#")[0]
			ops.object.select_name(name=item.split("#")[0]+"#"+item.split("#")[2], extend=True)
			lastnumber=item.split("#")[0]+"#"+item.split("#")[2]
		store.ChainModels[this_Protein+"#"+this_Chain]=lastnumber
		ops.group.create(name=lastnumber)
			
	except Exception as E:
		raise Exception ("Unable to write Aa list", E)
	ops.object.select_all(action="DESELECT")
	write_aa(to_List=aaList, verbose = False,kindOf="protein",selected="all")

def select_aminoacid(aaname):
	iteminfo=""
	try:
		for obj in context.scene.objects:
			if re.search("#",obj.name):
				splitname=obj.name.split("#")
				iteminfo=PDBString(obj.BBInfo).get("chainID")+" "+"%5.5d" % int(PDBString(obj.BBInfo).get("chainSeq")) + " " +PDBString(obj.BBInfo).get("aminoName")+" " +splitname[0]
				if aaname==iteminfo:
					ops.object.select_name(name=obj.name, extend=True)
	except Exception as E:
		raise Exception ("Unable to select aminoacid", E)
		return None
	else:
		return obj
		
def select_chain(this_chain):
	ops.object.select_all(action="DESELECT")
	iteminfo=""
	if this_chain!="all":
		if this_chain.split()[1]!="all":
			try:
				ops.object.select_all(action="DESELECT")
				ops.object.select_name(name=store.ChainModels[this_chain.split()[1]+"#"+this_chain.split()[0]], extend=False)
				ops.object.select_grouped(extend=False, type='GROUP')
			except Exception as E:
				raise Exception ("Unable to select a chain", E)
				return None
				
def write_aminoacids_for_protein(this_protein, verbose =  False):
	aaList = []
	#when a protein  is selected, walk through list of objects (atoms) in the scene. If the atom belongs to the protein, write aminoacid  chain, name, number and protein name in the list of aminoacids
	try:
		if this_protein=="all":
			for obj in context.scene.objects:
				if re.search("#",obj.name):
					splitname=obj.name.split("#")
					iteminfo=[splitname[0],PDBString(obj.BBInfo).get("chainID")+" "+"%5.5d" % int(PDBString(obj.BBInfo).get("chainSeq")) + " " +PDBString(obj.BBInfo).get("aminoName")]
					if not iteminfo in aaList:
						aaList.append(iteminfo)
			aaList.sort()
		else:
			for obj in context.scene.objects:
				if re.search("#",obj.name):
					splitname=obj.name.split("#")
					if splitname[0]==this_protein:
						iteminfo=[splitname[0],PDBString(obj.BBInfo).get("chainID")+" "+"%5.5d" % int(PDBString(obj.BBInfo).get("chainSeq")) + " " +PDBString(obj.BBInfo).get("aminoName")]
						if not iteminfo in aaList:
							aaList.append(iteminfo)
			aaList.sort()
	except Exception as E:
		raise Exception ("Unable to write aa list", E)
	write_aa(to_List=aaList, verbose = False,kindOf="protein",selected=this_protein)
	
def write_aminoacids_for_chain(this_chain, verbose =  False):
	print("writing for selected chain"+ this_chain)
	aaList = []
	#when a protein chain is selected, walk through list of objects (atoms) in the scene. If the atom belongs to the chain, write aminoacid  chain, name, number and protein name in the list of aminoacids
	try:
		if this_chain=="all":
			for obj in context.scene.objects:
				if re.search("#",obj.name):
					splitname=obj.name.split("#")
					iteminfo=[splitname[0],PDBString(obj.BBInfo).get("chainID")+" "+"%5.5d" % int(PDBString(obj.BBInfo).get("chainSeq")) + " " +PDBString(obj.BBInfo).get("aminoName")]
					if not iteminfo in aaList:
						aaList.append(iteminfo)
			aaList.sort()
		elif this_chain.split()[1]=="all":
			for obj in context.scene.objects:
				if re.search("#",obj.name):
					splitname=obj.name.split("#")
					if str(splitname[0])==this_chain.split()[0]:
						iteminfo=[splitname[0],PDBString(obj.BBInfo).get("chainID")+" "+"%5.5d" % int(PDBString(obj.BBInfo).get("chainSeq")) + " " +PDBString(obj.BBInfo).get("aminoName")]
						if not iteminfo in aaList:
							aaList.append(iteminfo)
			aaList.sort()
		else:
			for obj in context.scene.objects:
				if re.search("#",obj.name):
					splitname=obj.name.split("#")
					if splitname[0]==this_chain.split()[1]:
						if PDBString(obj.BBInfo).get("chainID")==this_chain.split()[0]:
							iteminfo=[splitname[0],PDBString(obj.BBInfo).get("chainID")+" "+"%5.5d" % int(PDBString(obj.BBInfo).get("chainSeq")) + " " +PDBString(obj.BBInfo).get("aminoName")]
							if not iteminfo in aaList:
								aaList.append(iteminfo)
			aaList.sort()
	except Exception as E:
		raise Exception ("Unable to write aa list", E)
	write_aa(to_List=aaList, verbose = False,kindOf="chain",selected=this_chain)

def restore_materials():
	try:
		for obj in context.scene.objects:
			if re.search("#",obj.name):
				index = PDBString(obj.BBInfo).get("element")
				obj.material_slots[0].material = data.materials[index]
	except Exception as E:
		raise Exception ("Unable to restore original materials", E)
def attribute_lipo_materials():
	try:
		for obj in context.scene.objects:
			if re.search("#",obj.name):
				aminoName = PDBString(obj.BBInfo).get("aminoName")
				#print("aminoName  "+str(aminoName))
				name = PDBString(obj.BBInfo).get("name")
				#print("name  "+str(name))
				#if name!="OXT":
				material_this=retrieve_fi_materials(am_name=aminoName,at_name=name)
				#print("mat_this  "+ str(material_this))
				obj.material_slots[0].material =data.materials[material_this]
	except Exception as E:
		raise Exception ("Unable to attribute lipo materials", E)




#I suppose that the object's referential was not change (the same as the scene, with x (red axis) to the right, z (blue axis) up and y (green axis) behing)
def addRigidBodyRotamer(objectparent,objecttarget) : 
	#Add the rigid body join for rotamer
	#to define a rotamer, an hinge is use, with the axis vector which come from the atom parent to the target and with a position at the center of the parent atom
	
	#This rotation transform the Ox axis of the parent, to the euler angle to orient the x axes (the hinge axis) of the pivot referential from this parent atom to the target  
	parentxaxis = Vector((1.0,0.0,0.0))
	hingevector=Vector((objecttarget.location[0]-objectparent.location[0], objecttarget.location[1]-objectparent.location[1], objecttarget.location[2]-objectparent.location[2]))
	rotvec2mapx2hingevector=parentxaxis.cross(hingevector)	
	rotvec2mapx2hingevector.normalize()
	angle2mapx2hingevector=parentxaxis.angle(hingevector)
	matrot=Matrix.Rotation(angle2mapx2hingevector, 3, rotvec2mapx2hingevector)
	euler=matrot.to_euler()

	#Add the rigid body join for rotamer	
	objectparent.constraints["RigidBody Joint"].target = objecttarget
	#objectparent.constraints["RigidBody Joint"].show_pivot = True
	
	objectparent.constraints["RigidBody Joint"].pivot_x = 0.0
	objectparent.constraints["RigidBody Joint"].pivot_y = 0.0
	objectparent.constraints["RigidBody Joint"].pivot_z = 0.0			
	
	objectparent.constraints["RigidBody Joint"].axis_x = euler[0]
	objectparent.constraints["RigidBody Joint"].axis_y = euler[1]
	objectparent.constraints["RigidBody Joint"].axis_z = euler[2]		
	
	#objectparent.rotation_mode='XYZ'

		